#include "SudokuCell.h"
#include "Sudoku.h"

#include <stdio.h>

#include <FL/fl_ask.H>
#include <FL/fl_draw.H>

class Sudoku;

// Create a cell widget
SudokuCell::SudokuCell(int X, int Y, int W, int H)
	: Fl_Widget(X, Y, W, H, 0)
{
  valueSet(0);
  uniquehintSet(0); //<1,9> as real values
  for (int i=0;i<10;i++)hints_[i]=0; 
}

// Draw cell
void
SudokuCell::draw() 
{
  static Fl_Align align[] = 
  {
  	(Fl_Align)0, /* value 0 never displayed */
    FL_ALIGN_TOP_LEFT,
    FL_ALIGN_TOP,
    FL_ALIGN_TOP_RIGHT,
    FL_ALIGN_LEFT,
    FL_ALIGN_CENTER,
    FL_ALIGN_RIGHT,
    FL_ALIGN_BOTTOM_LEFT,
    FL_ALIGN_BOTTOM,
    FL_ALIGN_BOTTOM_RIGHT
  };

  //determine background color
  if (!value_) //find "no hints in empty cell" background color
  { 
	  hints_[0] = 1;//no hints  
		for (int i = 1; i <= 9; i ++) 
		{ 
		    //if ((hints_[i] != i) && (hints_[i]!=0)) 
		    //	printf("cell draw::HINTS ERROR: cell(%d,%d) hint[%d]=%d\n", myI, myJ, i, hints_[i]); fflush(stdout);
		    if (hints_[i]) hints_[0] = 0; //hint found so no hints should be false;  
		}
	  //no hints background or restore
	  if (hints_[0]) 
	    color(FL_LIGHT2);  
	  else if (color() == FL_LIGHT2)
	  {  //reset the color
 		  if (readonlyGet())color(FL_GRAY);
      else color(FL_LIGHT3 );
	  }
  } 
     
  // Draw the cell box...
  if (readonlyGet()) 
    fl_draw_box(FL_UP_BOX, x(), y(), w(), h(), color());
  else 
    fl_draw_box(FL_DOWN_BOX, x(), y(), w(), h(), color());
    
  if (Fl::focus() == this) 
  {
    Fl_Color c = fl_color_average(FL_SELECTION_COLOR, color(), 0.5f);
    fl_color(c);
    fl_rectf(x() + 4, y() + 4, w() - 8, h() - 8);
    fl_color(fl_contrast(labelcolor(), c));
  } 
  else 
    fl_color(labelcolor());
    
  //string drawing helpers
  char s[2];
  s[1] = '\0';

  if (value_) 
  {
  	//Draw the cell value...
	  //char s[2];
  	s[1] = '\0';
    s[0] = value_ + '0'; //itoa

    fl_font(FL_HELVETICA_BOLD, h() - 10);
    fl_draw(s, x(), y(), w(), h(), FL_ALIGN_CENTER);
  }
  else
  {
	  //draw the hints only if no value
	  fl_color(FL_RED);
	  fl_font(FL_HELVETICA_BOLD, h() / 5);
	
	  for (int i = 1; i <= 9; i ++) 
	  { 
	    if (hints_[i]) 
	    {
	      //set drawing color
	      if (uniquehintGet() ==	hints_[i])
	      {
	      	fl_color(FL_BLACK);
	      }
	      s[0] = hints_[i] + '0';
	      fl_draw(s, x() + 5, y() + 5, w() - 10, h() - 10, align[i]);
	      
	      //afterward reset the drawing color back to red
	      if (uniquehintGet() == hints_[i])
	      {
	      	fl_color(FL_RED);
	      } 
	    }
	  }
  }
}// draw

// Handle events in cell
int
SudokuCell::handle(int event)
{
	int oldValue=0;
  //printf("event = %s\n", sdkcll_names[event]);	
  switch (event) 
  {
    case FL_FOCUS :
      Fl::focus(this);
      ((Sudoku *)window())->setFocusCell(myI,myJ);
      redraw();
      return 1;
      //case:FL_FOCUS
      
    case FL_UNFOCUS :
      redraw();
      return 1;
      break;
      //case:FL_UNFOCUS
      
    case FL_PUSH :
      //if (!readonly_ && Fl::event_inside(this)) //John improvements 
      //removed again to allow degrade for unique puzzle
      if (Fl::event_inside(this))  
      {
        if (Fl::event_clicks()) 
        {
		      // 2+ clicks increments/sets value
		      //DONE not increment but next (hint) value ; next_value(myI,myJ, currentValue)
		      //     nextvalue is different depending on auto hints
		      //DONE NOT for readonly cells
		      if (!readonly_)
		      {
		      	  oldValue = value_;
		        	valueSet(((Sudoku *)window())->next_value(myI,myJ));
		        	if (color() == FL_LIGHT2) color(FL_LIGHT3 );//reset possible light1
		      }//not readonly
		  	}
        Sudoku* s = (Sudoku *)window();
        s->setFocusCell(myI,myJ);

        Fl::focus(this);
       	//does a value so lets the whole value stuff work and update/redraw the whole grid
       	if ( (oldValue) && (s->getautohints()) ) s->restoreHint(oldValue, myI, myJ);
        if ( s->getautohints() ) s->updatehintsfromthis(myI, myJ);
        if ( s->getautounique() ) s->showuniquehints();
        if ( s->getpromoteunique() ) s->promoteuniquehints();
       	do_callback();//redraw whole grid
    	  return 1;
      }
      break;
      //case:FL_PUSH

    case FL_KEYDOWN :
      if (Fl::event_state() & FL_CTRL) break;
      
      int key = Fl::event_key() - '0';
      Sudoku* s = (Sudoku *)window();
      
      if (key < 0 || key > 9) key = Fl::event_key() - FL_KP - '0';//keypad keys to keys
      
      if (key > 0 && key <= 9) 
      { 
      	//cannot do actions on a puzzle value
        if (readonlyGet()) 
        {
          fl_beep(FL_BEEP_ERROR);
          return 1;
        }

        //handle as hints if shift or capslock
        if (Fl::event_state() & (FL_SHIFT | FL_CAPS_LOCK)) 
        {
		      if (hintsGet(key) )
    		  {
		      	//remove it exists
		   	    hintsRemove(key);
    		  }	
		      else
		      {
		  	    //does not exist write it
  	        hintsSet(key);
		      }
				  if ( s->getautounique() ) s->showuniquehints();
				  if ( s->getpromoteunique() ) s->promoteuniquehints();
				  do_callback();
			 	} 
				else 
				{
				  //add a user value	
				  oldValue = value_;
				  valueSet(key);
				  if (color() == FL_LIGHT2) color(FL_LIGHT3 );//reset possible light1
				  
				  if ( (oldValue) && (s->getautohints()) ) s->restoreHint(oldValue, myI, myJ); 
				  if ( s->getautohints() ) s->updatehintsfromthis(myI, myJ);
				  if ( s->getautounique() ) s->showuniquehints();
				  if ( s->getpromoteunique() ) s->promoteuniquehints();
				  do_callback();
				}
				return 1;
      } 
      else if (key == 0 || Fl::event_key() == FL_BackSpace ||
                 Fl::event_key() == FL_Delete) 
      {
      	//reset the value
        if (readonlyGet()) 
        {
          fl_beep(FL_BEEP_ERROR);
          return 1;
        }
        
        oldValue = value_;
        valueSet(0);
        if ( (oldValue) && (s->getautohints()) ) s->restoreHint(oldValue, myI, myJ); 
        if (s->getautohints()) s->hintthiscell(myI, myJ); 
			  if ( s->getautounique() ) s->showuniquehints();
  		  if ( s->getpromoteunique() ) s->promoteuniquehints();
			  do_callback();
	    	return 1;
      }
      /* other keys cell specific
      else if (Fl::event_key() =='h')
      {
      	//printf("cell handler h key pressed\n");
      	s->hintthiscell(myI, myJ);
      }
      else if (Fl::event_key() =='u')
      {
       	//printf("cell handler u key pressed [%d,%d]\n", myI, myJ);
       	s->updatehintsfromthis(myI, myJ);
      }            
      else if (Fl::event_key() == ' ')
      {      
       	s->cleanBackgrounds();
       	do_callback();
      }
      */
      break;
      //case:FL_KEYDOWN
  }
  return Fl_Widget::handle(event);
}// handle

void
SudokuCell::init(int x, int y)
{
	 myI = x;
	 myJ = y;
}// init

void
SudokuCell::printCell()
{
  printf("gridCell (%d,%d):%d:",myI,myJ,value_);
  for( int i=1; i<=9; i++)
 	{
	  printf("%d, ",hints_[i] );
	}
	printf(": %d\n", readonly_);
}// printCell

char	
SudokuCell::hintsGetChar(int n)
{
	if (hints_[n]) 
	  return '0'+ hints_[n];
	else
	  return '.';
}//hintsGetChat 
